unit PFontV13;

{********************************************************************
NAME: Po-Shen Loh.
TITLE: Po-Shen's font unit #1 (version 1.3).
PURPOSE: Draws characters, strings, and inputs data all in Po-Shen's
         Font #1.
CONTENTS: FontObject (font #1 procedures):
             Draw*(x coordinate, y coordinate, width in pixels, color(0..15).
             psClS(color(0..15).
             InitGraphLetts(ratio of height in pixels to the width in pixels,
                tab length, alarm bell frequency, path of graphics drivers,
                alarm bell time.
             psscanf(scan type, var: str, real, int, etc.). Returns a 1 if
                there is an error, and a 0 if there isn't.
             psprintf(x coordinate, y coordinate, width in pixels,
                color(0..15), font(in this ver. pass the value 1, char. str.).
             MoveTextCursor(x destination, y destination).
             GetTextCursorXY(vars: x coordinate, y coordinate).
             SetCurrColor(color (0..15)).
             GetCurrColor(var: color (0..15)).
             SetCurrSize(width in pixels).
             GetCurrSize(var: width in pixels).
             Destruct.

             FontObject is an object type that must be declared before it
                        may be used.

          Conversions:
             LongToString(long integer). Returns a string.
             IntToString(integer number). Returns a string.

TABLES:
       Color Table: 0 = black             8 = dark gray
                    1 = blue              9 = light blue
                    2 = green             10 = light green
                    3 = cyan              11 = light cyan
                    4 = red               12 = light red
                    5 = magenta           13 = light magenta
                    6 = brown             14 = yellow
                    7 = light gray        15 = white
       Scan Types:  %d = integer          %s = string
                    %l = long integer     %r = real
       Abbrev.:     SPC = space           EXC = !
                    NUM = #               QUES = ?
                    DLR = $               PRC = %
                    BSH = \               ODBQT = ``
                    CDBQT = ''            OSIQT = `
                    CSIQT = '             TIMES = x
                    AST = *               CPR = )
                    OPR = (               PLS = +
                    LES = <               GRE = >
                    LESOREQ = <=          GREOREQ = >=
                    APPROX = .            SIM = ~
                             =                   ~
                    EQU = =               OBR = [
                    CBR = ]               OBRACE = {
                    CBRACE = }     {      CAR = ^
                    UND = _               TIL = ~
                    CMA = ,               CLN = :
                    MNS = -               PRD = .
                    DIV = +               SLSH = /
                    SMICLN = ;

************************************************************}



{======================= Interface ========================}

interface

type paratype = array[1..5] of packed array[1..60] of char; {type def. used by flash para.}
     FontObject = object

     public  {start of public sector}

     procedure psClS(color : word);
     procedure Draw1(x, y : integer; size, color : word);
     procedure Draw2(x, y : integer; size, color : word);
     procedure Draw3(x, y : integer; size, color : word);
     procedure Draw4(x, y : integer; size, color : word);
     procedure Draw5(x, y : integer; size, color : word);
     procedure Draw6(x, y : integer; size, color : word);
     procedure Draw7(x, y : integer; size, color : word);
     procedure Draw8(x, y : integer; size, color : word);
     procedure Draw9(x, y : integer; size, color : word);
     procedure Draw0(x, y : integer; size, color : word);
     procedure Drawa(x, y : integer; size, color : word);
     procedure Drawz(x, y : integer; size, color : word);
     procedure Drawq(x, y : integer; size, color : word);
     procedure Draww(x, y : integer; size, color : word);
     procedure Draws(x, y : integer; size, color : word);
     procedure Drawx(x, y : integer; size, color : word);
     procedure Drawe(x, y : integer; size, color : word);
     procedure Drawd(x, y : integer; size, color : word);
     procedure Drawc(x, y : integer; size, color : word);
     procedure Drawr(x, y : integer; size, color : word);
     procedure Drawf(x, y : integer; size, color : word);
     procedure Drawv(x, y : integer; size, color : word);
     procedure Drawt(x, y : integer; size, color : word);
     procedure Drawg(x, y : integer; size, color : word);
     procedure Drawb(x, y : integer; size, color : word);
     procedure Drawy(x, y : integer; size, color : word);
     procedure Drawh(x, y : integer; size, color : word);
     procedure Drawn(x, y : integer; size, color : word);
     procedure Drawu(x, y : integer; size, color : word);
     procedure Drawj(x, y : integer; size, color : word);
     procedure Drawm(x, y : integer; size, color : word);
     procedure Drawi(x, y : integer; size, color : word);
     procedure Drawk(x, y : integer; size, color : word);
     procedure Drawo(x, y : integer; size, color : word);
     procedure Drawl(x, y : integer; size, color : word);
     procedure Drawp(x, y : integer; size, color : word);
     procedure DrawSPC(x, y : integer; size, color : word);
     procedure DrawEXC(x, y : integer; size, color : word);
     procedure DrawNUM(x, y : integer; size, color : word);
     procedure DrawQUES(x, y : integer; size, color : word);
     procedure DrawDLR(x, y : integer; size, color : word);
     procedure DrawPRC(x, y : integer; size, color : word);
     procedure DrawBSH(x, y : integer; size, color : word);
     procedure DrawODBQT(x, y : integer; size, color : word);
     procedure DrawCDBQT(x, y : integer; size, color : word);
     procedure DrawOSIQT(x, y : integer; size, color : word);
     procedure DrawCSIQT(x, y : integer; size, color : word);
     procedure DrawTIMES(x, y : integer; size, color : word);
     procedure DrawAST(x, y : integer; size, color : word);
     procedure DrawCPR(x, y : integer; size, color : word);
     procedure DrawOPR(x, y : integer; size, color : word);
     procedure DrawPLS(x, y : integer; size, color : word);
     procedure DrawLES(x, y : integer; size, color : word);
     procedure DrawGRE(x, y : integer; size, color : word);
     procedure DrawLESOREQ(x, y : integer; size, color : word);
     procedure DrawGREOREQ(x, y : integer; size, color : word);
     procedure DrawEQU(x, y : integer; size, color : word);
     procedure DrawSIM(x, y : integer; size, color : word);
     procedure DrawAPPROX(x, y : integer; size, color : word);
     procedure DrawOBR(x, y : integer; size, color : word);
     procedure DrawCBR(x, y : integer; size, color : word);
     procedure DrawCAR(x, y : integer; size, color : word);
     procedure DrawUND(x, y : integer; size, color : word);
     procedure DrawOBRACE(x, y : integer; size, color : word);
     procedure DrawCBRACE(x, y : integer; size, color : word);
     procedure DrawTIL(x, y : integer; size, color : word);
     procedure DrawCMA(x, y : integer; size, color : word);
     procedure DrawCLN(x, y : integer; size, color : word);
     procedure DrawMNS(x, y : integer; size, color : word);
     procedure DrawPRD(x, y : integer; size, color : word);
     procedure DrawDIV(x, y : integer; size, color : word);
     procedure DrawSLSH(x, y : integer; size, color : word);
     procedure DrawSMICLN(x, y : integer; size, color : word);
     procedure Init(YDX : real; tabl, abf, abt : integer);
     procedure psprintf(x, y, siz, col, font : integer; chrstr : string);
     procedure psflashpara(x, y, siz, col, xin, yin : integer; rows, cols : byte; delaytime : integer;
                              tail_length : integer; para : paratype);
     function psscanf(scantype : string; var ptr) : byte;
     function psFlashScanf(var returnstr : string; timelapse : word; x, y : integer; siz, col : word) : integer;
     procedure movetextcursor(x, y : integer);
     procedure GetTextCursorXY(var x, y : integer);
     procedure SetCurrColor(color : word);
     procedure GetCurrColor(var color : word);
     procedure SetCurrSize(size : word);
     procedure GetCurrSize(var size : word);
     procedure Destruct;

     private     {start of private sector}

     textcursorx, textcursory : integer;
     currentcolor, currentsize : word;
     YDIVX : real;
     tablength, alarmbellfrequency, alarmbelltime : integer;

     end;   {object}

function IntToString(inttoconvert : integer) : string;  {converts integers to strings}
function LongToString(longtostrnum : longint) : string; {converts longints to strings}
procedure pGetTime(var h, m, s, s100 : longint);

{========================== Implementation ==============================}

implementation

uses graph,
     crt,
     DOS,
     pBGIdriv,
     pBGIfont;  {includes these five units}

procedure pGetTime(var h, m, s, s100 : longint);

var
   h2, m2, s2, s1002 : word;

begin
     gettime(h2, m2, s2, s1002);
     h := h2;
     m := m2;
     s := s2;
     s100 := s1002
end;

{~~~~~~~~~~~~~~~~~~~ Beginning of FontObject ~~~~~~~~~~~~~~~~~~~~~~}


{************************ PsClearScr ******************************}
{clears the screen, with the background color that was passed}

procedure FontObject.psClS(color : word);

begin {procedure}
     setfillstyle(SolidFill, color); {sets the fill color to the one passed}
     setcolor(color);                {sets the draw color to the one passed}
     bar(0, 0, GetMaxX, GetMaxY);    {draws a huge rectangle around the screen}
     MoveTextCursor(2, 2);           {moves text cursor to (2, 2)}
end;  {procedure}

{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
            CHARACTER DRAWING ROUTINES START HERE
            ALL CHARS ARE DRAWN WITH LINES AND/OR
            ARCS.
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}

{*********************** FontObject.Draw1 **************************}

procedure FontObject.Draw1(x, y : integer; size, color : word);

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX), x + size, trunc(y + size * YDIVX));
     line(x + size div 2, y, x + size div 2, trunc(y + size * YDIVX));
     line(x, trunc(y + size * YDIVX / 6), x + size div 2, y);
end; {Draw1}

{************************* Draw2 ****************************}

procedure FontObject.Draw2(x, y : integer; size, color : word);

begin
     setcolor(color);
     arc(x + size div 2, y + size div 2, trunc(330 - 2 * (ArcTan(2 * YDIVX - 1))), 180, size div 2);
     moveto(x, trunc(y + size * YDIVX));
     linerel(round((size*(YDIVX-1/2)*sin(2*ArcTan(2))))+round(size/7)-1,round((size*(YDIVX-1/2)*cos(2 * ArcTan(2)))));
     line(x, trunc(y + size * YDIVX), x + size, trunc(y + size * YDIVX));
end; {Draw2}

{*********************** Draw3 **************************}

procedure FontObject.Draw3(x, y : integer; size, color : word);

begin
     setcolor(color);
     ellipse(x + size div 2, trunc(y + size * YDIVX / 4), 270, 180, size div 2, trunc(size * YDIVX / 4));
     ellipse(x + size div 2, trunc(y + size * 3 * YDIVX / 4), 180, 90, size div 2, trunc(size * YDIVX / 4));
end; {Draw3}

{************************ Draw4 ****************************}

procedure FontObject.Draw4(x, y : integer; size, color : word);

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX / 2), x + size, trunc(y + size * YDIVX / 2));
     line(x + 3 * size div 4, y, x + 3 * size div 4, trunc(y + size * YDIVX));
     line(x + 3 * size div 4, y, x, trunc(y + size * YDIVX / 2));
end; {Draw4}

{*********************** Draw5 ***************************}

procedure FontObject.Draw5(x, y : integer; size, color : word);

begin
     setcolor(color);
     ellipse(x + size div 2, trunc(y + size * 3 * YDIVX / 4), 180, 90, size div 2, trunc(size * YDIVX / 4));
     line(x, y, x, trunc(y + size * YDIVX / 2));
     line(x, y, x + size, y);
     line(x, trunc(y + size * YDIVX / 2), x + size div 2, trunc(y + size * YDIVX / 2));
end; {Draw5}

{*********************** Draw6 ******************************}

procedure FontObject.Draw6(x, y : integer; size, color : word);

begin
     setcolor(color);
     ellipse(x + size div 2, trunc(y + size * 3 * YDIVX / 4), 0, 360, size div 2, trunc(size * YDIVX / 4));
     arc(trunc(x + size / 2), trunc(y + size / 2), 90, 180, trunc(size / 2));
     line(x, trunc(y + size * YDIVX / 4), x, trunc(y + 3 * size * YDIVX / 4));
end; {Draw6}

{*********************** Draw7 *****************************}

procedure FontObject.Draw7;

begin
     setcolor(color);
     line(x, y, x + size, y);
     line(x, trunc(y + size * YDIVX), x + size, y);
end; {Draw7}

{*********************** Draw8 ********************************}

procedure FontObject.Draw8;

begin
     setcolor(color);
     ellipse(x + size div 2, trunc(y + size * YDIVX / 4), 0, 360, size div 2, trunc(size * YDIVX / 4));
     ellipse(x + size div 2, trunc(y + size * 3 * YDIVX / 4), 0, 360, size div 2, trunc(size * YDIVX / 4));
end; {Draw8}

{*********************** Draw9 *******************************}

procedure FontObject.Draw9;

begin
     setcolor(color);
     ellipse(x + size div 2, trunc(y + size * YDIVX / 4), 0, 360, size div 2, trunc(size * YDIVX / 4));
     line(x + size div 2, trunc(y + size * YDIVX), x + size, trunc(y + size * YDIVX / 4));
end; {Draw9}

{**************************** Draw0 **********************************}

procedure FontObject.Draw0;

begin
     setcolor(color);
     ellipse(x + size div 2, trunc(y + size * YDIVX / 2), 0, 360, size div 2, trunc(size * YDIVX / 2));
end; {Draw0}

{********************** DrawSPC ****************************}

procedure FontObject.DrawSPC;

var
   par : paratype;

begin
     if false then begin
        par[1] := 'This unit was created by Po-Shen Loh!                       ';
        psflashpara(5, 200, 8, 2, 12, 20, 1, 40, 30, 3, par);
     end
     else begin
          setfillstyle(solidfill, 0);
          bar(x, y, x + size, y + trunc(size * YDIVX));
     end
end;

{********************** DrawEXC ***************************}

procedure FontObject.DrawEXC;

begin
     setcolor(color);
     line(x + size div 2, y, x + size div 2, trunc(y + 2 * size * YDIVX / 3));
     putpixel(x + size div 2, trunc(y + size * YDIVX), color);
end; {DrawEXC}

{********************** Draw# *************************}

procedure FontObject.DrawNUM;

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX / 3), x +  size, trunc(y + size * YDIVX / 3));
     line(x, trunc(y + 2 * size * YDIVX / 3), x +  size, trunc(y + 2 * size * YDIVX / 3));
     line(x + size div 3, y, x + size div 3, trunc(y + size * YDIVX));
     line(x + 2 * size div 3, y, x + 2 * size div 3, trunc(y + size * YDIVX));
end; {Draw#}

{************************* DrawQUES *****************************}

procedure FontObject.DrawQUES;

begin
     setcolor(color);
     arc(trunc(x + size / 2), trunc(y + size / 2), 0, 180, trunc(size / 2));
     ellipse(x + 3 * size div 4, y + size div 2, 270, 360, size div 4, trunc(size * YDIVX / 2 - size / 2));
     ellipse(x+3*size div 4, y+size div 2+2*trunc(size*YDIVX/2-size/2), 90, 180, size div 4, trunc(size*YDIVX/2-size/2));
     line(x + size div 2, y + size div 2 + 2 * trunc(size*YDIVX/2-size/2), x + size div 2, trunc(y + 5 / 6 * size * YDIVX));
     PutPixel(x + size div 2, trunc(y + size * YDIVX), color);
end; {DrawQUES}

{************************ DrawDLR ****************************}

procedure FontObject.DrawDLR;

begin
     DrawS(x + 1, y + 1, size - 2, color);
     line(x + size div 2, y, x + size div 2, trunc(y + size * YDIVX));
end; {DrawDLR}

{************************* DrawPRC ***************************}

procedure FontObject.DrawPRC;

begin
     setcolor(color);
     line(x + size, y, x, trunc(y + size * YDIVX));
     arc(x + size div 8, y + size div 8, 0, 360, size div 8);
     arc(x + 7 * size div 8, trunc(y + size * YDIVX - size div 8), 0, 360, size div 8);
end; {DrawPRC}

{************************* DrawBSH ***************************************}

procedure FontObject.DrawBSH;

begin
     setcolor(color);
     line(x, y, x + size, trunc(y + size * YDIVX));
end; {DrawBSH}

{************************* Draw" ***********************************}

procedure FontObject.DrawODBQT;

begin
     setcolor(color);
     setfillstyle(SolidFIll, color);
     circle(x + size div 4, y + size div 8, size div 8);
     floodfill(x + size div 4, y + size div 8, color);
     circle(x + 3 * size div 4, y + size div 8, size div 8);
     floodfill(x + 3 * size div 4, y + size div 8, color);
     ellipse(x + 3 * size div 8 - 1, y + size div 8 + 1, 180, 270, size div 8, trunc(size * YDIVX / 6));
     ellipse(x + 7 * size div 8 - 1, y + size div 8 + 1, 180, 270, size div 8, trunc(size * YDIVX / 6));

end;

procedure FontObject.DrawCDBQT;

begin
     setcolor(color);
     setfillstyle(SolidFill, color);
     circle(x + size div 4, y + size div 6 + 1 + trunc(size * YDIVX / 6) - size div 8, size div 8);
     floodfill(x + size div 4, y + size div 6 + 1 + trunc(size * YDIVX / 6) - size div 8, color);
     circle(x + 3 * size div 4, y + size div 6 + 1 + trunc(size * YDIVX / 6) - size div 8, size div 8);
     floodfill(x + 3 * size div 4, y + size div 6 + 1 + trunc(size * YDIVX / 6) - size div 8, color);
     ellipse(x + size div 4, y + size div 6, 330, 90, size div 8, trunc(size * YDIVX / 6));
     ellipse(x + 3 * size div 4, y + size div 8, 330, 90, size div 8, trunc(size * YDIVX / 6));

end;

procedure FontObject.DrawOSIQT;

begin
     setcolor(color);
     setfillstyle(SolidFill, color);
     circle(x + size div 2, y + size div 8, size div 8);
     floodfill(x + size div 2, y + size div 8, color);
     ellipse(x + 5 * size div 8 - 1, y + size div 8 + 1, 180, 270, size div 8, trunc(size * YDIVX / 6));
end;

procedure FontObject.DrawCSIQT;

begin
     setcolor(color);
     setfillstyle(SolidFill, color);
     circle(x + size div 2, y + size div 6 + 1 + trunc(size * YDIVX / 6) - size div 8, size div 8);
     floodfill(x + size div 2, y + size div 6 + 1 + trunc(size * YDIVX / 6) - size div 8, color);
     ellipse(x + size div 2, y + size div 8, 330, 90, size div 8, trunc(size * YDIVX / 6));
end;

{*****************************  (..) ***************************}

procedure FontObject.DrawOPR;

begin
     setcolor(color);
     ellipse(x + 3 * size div 4, trunc(y + size / 2 * YDIVX), 90, 270, size div 2, trunc(size / 2 * YDIVX));
end;

procedure FontObject.DrawCPR;

begin
     setcolor(color);
     ellipse(x + 3 * size div 4, trunc(y + size / 2 * YDIVX), 270, 90, size div 2, trunc(size / 2 * YDIVX));
end;

{****************************** Draw *, X ********************************}

procedure FontObject.DrawAST;

begin
     setcolor(color);
     line(x + size div 4, trunc(y + size / 2 * YDIVX), x + 3 * size div 4, trunc(y + size / 2 * YDIVX));
     moveto(x + size div 2, trunc(y + size * YDIVX / 2));
     moverel(size div 8, -1 * trunc(sqrt(3) * size / 8));
     linerel(-1 * size div 4, 2 * trunc(sqrt(3) * size / 8));
     moveto(x + size div 2, trunc(y + size * YDIVX / 2));
     moverel(-1 * size div 8, -1 * trunc(sqrt(3) * size / 8));
     linerel(size div 4, 2 * trunc(sqrt(3) * size / 8));
end;

procedure FontObject.DrawTIMES;

begin
     setcolor(color);
     line(x + size div 8, trunc(y + size / 2 * YDIVX - 3 * size / 8), x + 7 * size div 8, trunc(y + size/2*YDIVX+3*size/8));
     line(x + 7 * size div 8, trunc(y + size / 2 * YDIVX - 3 * size / 8), x + size div 8, trunc(y + size/2*YDIVX+3*size/8));
end;

{************************ DrawPLS ******************************}

procedure FontObject.DrawPLS;

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX / 2), x + size, trunc(y + size * YDIVX / 2));
     line(x + size div 2, trunc(y + size * YDIVX / 2 - size / 2), x + size div 2, trunc(y + size * YDIVX / 2 + size / 2));
end; {DrawPLS}

{************************ DrawCMA ******************************}

procedure FontObject.DrawCMA;

begin
     setcolor(color);
     setfillstyle(SolidFill, color);
     circle(x + size div 2, trunc(y + size * YDIVX - size / 8), size div 8);
     floodfill(x + size div 2, trunc(y + size * YDIVX - size / 8), color);
     ellipse(x + 5 * size div 8 - 1, trunc(y + size * YDIVX + size div 8 - 1), 270, 90, size div 8, trunc(size * YDIVX / 6));
end;

{************************** DrawMNS ****************************}

procedure FontObject.DrawMNS;

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX / 2), x + size, trunc(y + size * YDIVX / 2));
end; {DrawMNS}

{*********************** DrawPRD *************************}

procedure FontObject.DrawPRD;

begin
     setcolor(color);
     setfillstyle(SolidFill, color);
     circle(x + size div 2, trunc(y + size * YDIVX - size / 8), size div 8);
     floodfill(x + size div 2, trunc(y + size * YDIVX - size / 8), color);
end;

{*********************** Draw /,  DIV *****************************}

procedure FontObject.DrawSLSH;

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX), x + size, y);
end; {DrawSLSH}

procedure FontObject.DrawDIV;

begin
     DrawMNS(x, y, size, color);
     circle(x + size div 2, trunc(y + 3/4 * size * YDIVX), size div 8);
     circle(x + size div 2, trunc(y + 1/4 * size * YDIVX), size div 8);
     setfillstyle(SolidFill, color);
     floodfill(x + size div 2, trunc(y + 3/4 * size * YDIVX), color);
     floodfill(x + size div 2, trunc(y + 1/4 * size * YDIVX), color);
end; {DrawDIV}

{******************** Draw CLN, SMICLN *******************************}

procedure FontObject.DrawCLN;

begin
     setcolor(color);
     circle(x + size div 2, trunc(y + 3/4 * size * YDIVX), size div 8);
     circle(x + size div 2, trunc(y + 1/4 * size * YDIVX), size div 8);
     setfillstyle(SolidFill, color);
     floodfill(x + size div 2, trunc(y + 3/4 * size * YDIVX), color);
     floodfill(x + size div 2, trunc(y + 1/4 * size * YDIVX), color);
end;

procedure FontObject.DrawSMICLN;

begin
     setcolor(color);
     circle(x + size div 2, trunc(y + 1/4 * size * YDIVX), size div 8);
     setfillstyle(SolidFill, color);
     floodfill(x + size div 2, trunc(y + 1/4 * size * YDIVX), color);
     DrawCMA(x, trunc(y - size * YDIVX / 4 + size / 8), size, color);
end; {DrawSMICLN}

{******************* DrawLES **********************}

procedure FontObject.DrawLES;

begin
     setcolor(color);
     line(x + size, trunc(y + size * YDIVX / 4), x, trunc(y + size * YDIVX / 2));
     line(x + size, trunc(y + 3 * size * YDIVX / 4), x, trunc(y + size * YDIVX / 2));
end;

procedure FontObject.DrawGRE;

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX / 4), x + size, trunc(y + size * YDIVX / 2));
     line(x, trunc(y + 3 * size * YDIVX / 4), x + size, trunc(y + size * YDIVX / 2));
end;

procedure FontObject.DrawLESOREQ;

begin
     DrawLES(x, y, size, color);
     line(x + size, trunc(y + 3 * size * YDIVX / 4 + 1), x, trunc(y + size * YDIVX / 2 + 1));
end;

procedure FontObject.DrawGREOREQ;

begin
     DrawGRE(x, y, size, color);
     line(x, trunc(y + 3 * size * YDIVX / 4 + 1), x + size, trunc(y + size * YDIVX / 2 + 1));
end;

procedure FontObject.DrawEQU;

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX / 2 - 1), x + size, trunc(y + size * YDIVX / 2 - 1));
     line(x, trunc(y + size * YDIVX / 2 + 1), x + size, trunc(y + size * YDIVX / 2 + 1));
end;

procedure FontObject.DrawSIM;          {\~}

begin
     setcolor(color);
     ellipse(x + size div 4, trunc(y + size * YDIVX / 2), 180, 360, size div 4, trunc(size * YDIVX / 8));
     ellipse(x + 3 * size div 4, trunc(y + size * YDIVX / 2), 0, 180, size div 4, trunc(size * YDIVX / 8));
end;

procedure FontObject.DrawAPPROX;

begin
     setcolor(color);
     ellipse(x + size div 4, trunc(y + size * YDIVX / 2 - 1), 180, 360, size div 4, trunc(size * YDIVX / 8));
     ellipse(x + 3 * size div 4, trunc(y + size * YDIVX / 2 - 1), 0, 180, size div 4, trunc(size * YDIVX / 8));
     ellipse(x + size div 4, trunc(y + size * YDIVX / 2 + 1), 180, 360, size div 4, trunc(size * YDIVX / 8));
     ellipse(x + 3 * size div 4, trunc(y + size * YDIVX / 2 + 1), 0, 180, size div 4, trunc(size * YDIVX / 8));
end;

{******************* Draw Brackets ******************}

procedure FontObject.DrawOBR;

begin
     setcolor(color);
     line(x + 3 * size div 8, y, x + 3 * size div 8, trunc(y + size * YDIVX));
     line(x + 3 * size div 8, y, x + 5 * size div 8, y);
     line(x + 3 * size div 8, trunc(y + size * YDIVX), x + 5 * size div 8, trunc(y + size * YDIVX));
end;

procedure FontObject.DrawCBR;

begin
     setcolor(color);
     line(x + 5 * size div 8, y, x + 5 * size div 8, trunc(y + size * YDIVX));
     line(x + 3 * size div 8, y, x + 5 * size div 8, y);
     line(x + 3 * size div 8, trunc(y + size * YDIVX), x + 5 * size div 8, trunc(y + size * YDIVX));
end;

{********************* Draw Carat **********************}

procedure FontObject.DrawCAR;

begin
     setcolor(color);
     line(x + size div 2, y, x + size, trunc(y + size * YDIVX * 3/8));
     line(x + size div 2, y, x, trunc(y + size * YDIVX * 3/8));
end;

{******************** Draw Underscore ************************}

procedure FontObject.DrawUND;

begin
     setcolor(color);
     line(x, trunc(y + size * YDIVX), x + size, trunc(y + size * YDIVX));
end;

{********************** Draw Braces *************************}

procedure FontObject.DrawOBRACE;

begin
     setcolor(color);
     ellipse(x + 5 * size div 8, trunc(y + size * YDIVX / 4), 90, 180, size div 8, trunc(y + size * YDIVX / 4));
     ellipse(x + 5 * size div 8, trunc(y + 3 * size * YDIVX / 4), 180, 270, size div 8, trunc(y + size * YDIVX / 4));
     arc(x + 3 * size div 8, trunc(y + size * YDIVX / 2 - size / 8), 270, 360, size div 8);
     arc(x + 3 * size div 8, trunc(y + size * YDIVX / 2 + size / 8), 0, 90, size div 8);
     line(x + size div 2, trunc(y + size * YDIVX / 4), x + size div 2, trunc(y + size * YDIVX / 2 - size / 8));
     line(x + size div 2, trunc(y + 3 * size * YDIVX / 4), x + size div 2, trunc(y + size * YDIVX / 2 + size / 8));
end;

procedure FontObject.DrawCBRACE;

begin
     setcolor(color);
     ellipse(x + 3 * size div 8, trunc(y + size * YDIVX / 4), 0, 90, size div 8, trunc(y + size * YDIVX / 4));
     ellipse(x + 3 * size div 8, trunc(y + 3 * size * YDIVX / 4), 270, 360, size div 8, trunc(y + size * YDIVX / 4));
     arc(x + 5 * size div 8, trunc(y + size * YDIVX / 2 - size / 8), 270, 360, size div 8);
     arc(x + 5 * size div 8, trunc(y + size * YDIVX / 2 + size / 8), 0, 90, size div 8);
     line(x + size div 2, trunc(y + size * YDIVX / 4), x + size div 2, trunc(y + size * YDIVX / 2 - size / 8));
     line(x + size div 2, trunc(y + 3 * size * YDIVX / 4), x + size div 2, trunc(y + size * YDIVX / 2 + size / 8));
end;

{********************** Draw Tilde **************************}

procedure FontObject.DrawTIL;

begin
     setcolor(color);
     ellipse(x + size div 4, trunc(size * YDIVX / 8), 180, 360, size div 4, trunc(size * YDIVX / 8));
     ellipse(x + 3 * size div 4, trunc(size * YDIVX / 8), 0, 180, size div 4, trunc(size * YDIVX / 8));
end;

{*********************** DrawA **************************}

procedure FontObject.DrawA(x, y: integer; size, color : word);

var
   par : paratype;

begin
     if true then begin
        setcolor(color);
        arc(trunc(x + size / 2), trunc(y + size / 2), 0, 180, trunc(size / 2));
        line(x, trunc(y + size / 2), x, trunc(y + YDIVX * size));
        line(x + size, trunc(y + size / 2), x + size, trunc(y + YDIVX * size));
        line(x, trunc(y + YDIVX * size / 2), x + size, trunc(y + YDIVX * size / 2))
     end
     else begin
          par[1] := 'This unit was created by Po-Shen Loh!                       ';
          psflashpara(5, 200, 8, 2, 12, 20, 1, 40, 30, 3, par);
     end;
end; {DrawA}

{*********************** DrawB ****************************}

procedure FontObject.DrawB;

begin
     setcolor(color);
     arc(trunc(x + size - (size * YDIVX / 4)), trunc(y + (size * YDIVX / 4)), 270, 90, trunc(size * YDIVX / 4));
     arc(trunc(x + size - (size * YDIVX / 4)), trunc(y + 3 * (size * YDIVX / 4)), 270, 90, trunc(size * YDIVX / 4));
     line(x, y, x, trunc(y + size * YDIVX));
     line(x, y, trunc(x + size - (size * YDIVX / 4)), y);
     line(x, trunc(y + YDIVX * size / 2), trunc(x + size - (size * YDIVX / 4)), trunc(y + YDIVX * size / 2));
     line(x, trunc(y + size * YDIVX), trunc(x + size - (size * YDIVX / 4)), trunc(y + size * YDIVX));
end; {DrawB}

{******************** DrawC *****************************}

procedure FontObject.DrawC;

begin
     setcolor(color);
     arc(trunc(x + size / 2), trunc(y + size / 2), 30, 180, trunc(size / 2));
     arc(trunc(x + size / 2), trunc(y + size * YDIVX - size / 2), 180, 330, trunc(size / 2));
     line(x, y + size div 2, x, trunc(y + size * YDIVX - size / 2));
end; {DrawC}

{******************** DrawD *****************************}

procedure FontObject.DrawD;

begin
     setcolor(color);
     arc(trunc(x + size - (size * YDIVX / 4)), trunc(y + (size * YDIVX / 4)), 0, 90, trunc(size * YDIVX / 4));
     arc(trunc(x + size - (size * YDIVX / 4)), trunc(y + 3 * (size * YDIVX / 4)), 270, 0, trunc(size * YDIVX / 4));
     line(trunc(x+size), trunc(y+size*YDIVX/4), trunc(x+size), trunc(y+YDIVX*size-size*YDIVX/4));
     line(x, y, x, trunc(y + size * YDIVX));
     line(x, y, trunc(x + size - (size * YDIVX / 4)), y);
     line(x, trunc(y + size * YDIVX), trunc(x + size - (size * YDIVX / 4)), trunc(y + size * YDIVX));
end; {DrawD}

{************************** DrawE *****************************}

procedure FontObject.DrawE;

var
   par : paratype;

begin
     if true then begin
        setcolor(color);
        line(x, y, x, trunc(y + size * YDIVX));
        line(x, y, x + size, y);
        line(x, trunc(y + size * YDIVX / 2), trunc(x + 5 / 6 * size), trunc(y + size * YDIVX / 2));
        line(x, trunc(y + size * YDIVX), x + size, trunc(y + size * YDIVX))
     end
     else begin
          par[1] := 'This unit was created by Po-Shen Loh!                       ';
          psflashpara(5, 200, 8, 2, 12, 20, 1, 40, 30, 3, par);
     end;
end; {DrawE}

{************************* DrawF ****************************}

procedure FontObject.DrawF;

begin
     setcolor(color);
     line(x, y, x, trunc(y + size * YDIVX));
     line(x, y, x + size, y);
     line(x, trunc(y + size * YDIVX / 2), trunc(x + 5 / 6 * size), trunc(y + size * YDIVX / 2));
end; {DrawF}

{***************************** DrawG ********************************}

procedure FontObject.DrawG;

begin
     setcolor(color);
     arc(trunc(x + size / 2), trunc(y + size / 2), 30, 180, trunc(size / 2));
     arc(trunc(x + size / 2), trunc(y + size * YDIVX - size / 2), 180, 360, trunc(size / 2));
     line(x, y + size div 2, x, trunc(y + size * YDIVX - size / 2));
     line(x + size, trunc(y + size * YDIVX - size / 2), x + size, trunc(y + size * YDIVX / 2));
     line(x + size div 2, trunc(y + YDIVX * size / 2), x + size, trunc(y + YDIVX * size / 2));
end; {DrawG}

{**************************** DrawH **********************************}

procedure FontObject.DrawH;

begin
     setcolor(color);
     line(x, y, x, trunc(y + size * YDIVX));
     line(x + size, y, x + size, trunc(y + size * YDIVX));
     line(x, trunc(y + size * YDIVX / 2), trunc(x + size), trunc(y + size * YDIVX / 2));
end; {DrawH}

{************************* DrawI **************************}

procedure FontObject.DrawI;

begin
     setcolor(color);
     line(x, y, x + size, y);
     line(x, trunc(y + size * YDIVX), x + size, trunc(y + size * YDIVX));
     line(x + size div 2, y, x + size div 2, trunc(y + size * YDIVX));
end; {DrawI}

{*********************** DrawJ ************************}

procedure FontObject.DrawJ;

begin
     setcolor(color);
     arc(trunc(x + size / 2), trunc(y + size * YDIVX - size / 2), 180, 360, trunc(size / 2));
     line(x + size, y, x + size, trunc(y + size * YDIVX - size / 2));
     line(x, trunc(y + size * YDIVX / 2), x, trunc(y + size * YDIVX - size / 2));
end; {DrawJ}

{********************** DrawK *************************}

procedure FontObject.DrawK;

begin
     setcolor(color);
     line(x, y, x, trunc(y + size * YDIVX));
     line(x, trunc(y + size * YDIVX / 2), x + size, y);
     line(x, trunc(y + size * YDIVX / 2), x + size, trunc(y + size * YDIVX));
end; {DrawK}

{********************* DrawL **************************}

procedure FontObject.DrawL;

begin
     setcolor(color);
     line(x, y, x, trunc(y + size * YDIVX));
     line(x, trunc(y + size * YDIVX), x + size, trunc(y + size * YDIVX));
end; {DrawL}

{********************* DrawM ****************************}

procedure FontObject.DrawM;

begin
     setcolor(color);
     line(x, y, x, trunc(y + size * YDIVX));
     line(x + size, y, x + size, trunc(y + size * YDIVX));
     line(x, y, x + size div 2, trunc(y + size * YDIVX / 2));
     line(x + size, y, x + size div 2, trunc(y + size * YDIVX / 2));
end; {DrawM}

{************************* DrawN ******************************}

procedure FontObject.DrawN;

begin
     setcolor(color);
     line(x, y, x, trunc(y + size * YDIVX));
     line(x + size, y, x + size, trunc(y + size * YDIVX));
     line(x, y, x + size, trunc(y + size * YDIVX));
end; {DrawN}

{************************* DrawO *********************************}

procedure FontObject.DrawO;

var
   par : paratype;

begin
     if true then begin
        setcolor(color);
        arc(trunc(x + size / 2), trunc(y + size / 2), 0, 180, trunc(size / 2));
        arc(trunc(x + size / 2), trunc(y + size * YDIVX - size / 2), 180, 360, trunc(size / 2));
        line(x, y + size div 2, x, trunc(y + size * YDIVX - size / 2));
        line(x + size, y + size div 2, x + size, trunc(y + size * YDIVX - size / 2))
     end
     else begin
          par[1] := 'This unit was created by Po-Shen Loh!                       ';
          psflashpara(5, 200, 8, 2, 12, 20, 1, 40, 30, 3, par);
     end;
end; {DrawO}

{******************** DrawP **************************}

procedure FontObject.DrawP;

begin
     setcolor(color);
     arc(trunc(x + size - (size * YDIVX / 4)), trunc(y + (size * YDIVX / 4)), 270, 90, trunc(size * YDIVX / 4));
     line(x, y, x, trunc(y + size * YDIVX));
     line(x, y, trunc(x + size - (size * YDIVX / 4)), y);
     line(x, trunc(y + YDIVX * size / 2), trunc(x + size - (size * YDIVX / 4)), trunc(y + YDIVX * size / 2));
end; {DrawP}

{******************** DrawQ ****************************}

procedure FontObject.DrawQ;

begin
     setcolor(color);
     arc(trunc(x + size / 2), trunc(y + size / 2), 0, 180, trunc(size / 2));
     arc(trunc(x + size / 2), trunc(y + size * YDIVX - size / 2), 180, 360, trunc(size / 2));
     line(x, y + size div 2, x, trunc(y + size * YDIVX - size / 2));
     line(x + size, y + size div 2, x + size, trunc(y + size * YDIVX - size / 2));
     line(trunc(x + size / 2), trunc(y + size * YDIVX - size / 2), x + size, trunc(y + size * YDIVX));
end; {DrawQ}

{******************** DrawR **************************}

procedure FontObject.DrawR;

begin
     setcolor(color);
     arc(trunc(x + size - (size * YDIVX / 4)), trunc(y + (size * YDIVX / 4)), 270, 90, trunc(size * YDIVX / 4));
     line(x, y, x, trunc(y + size * YDIVX));
     line(x, y, trunc(x + size - (size * YDIVX / 4)), y);
     line(x, trunc(y + YDIVX * size / 2), trunc(x + size - (size * YDIVX / 4)), trunc(y + YDIVX * size / 2));
     line(x, trunc(y + size * YDIVX / 2), x + size, trunc(y + size * YDIVX));
end; {DrawR}

{*********************** DrawS **************************}

procedure FontObject.DrawS;

begin
     setcolor(color);
     ellipse(x + size div 2, trunc(y + size * YDIVX / 4), 0, 270, size div 2, trunc(size * YDIVX / 4));
     ellipse(x + size div 2, trunc(y + size * 3 * YDIVX / 4), 180, 90, size div 2, trunc(size * YDIVX / 4));
end; {DrawS}

{*********************** DrawT **************************}

procedure FontObject.DrawT;

var
   par : paratype;

begin
     if true then begin
        setcolor(color);
        line(x, y, x + size, y);
        line(x + size div 2, y, x + size div 2, trunc(y + size * YDIVX))
     end
     else begin
          par[1] := 'This unit was created by Po-Shen Loh!                       ';
          psflashpara(5, 200, 8, 2, 12, 20, 1, 40, 30, 3, par);
     end;
end; {DrawT}

{*********************** DrawU **************************}

procedure FontObject.DrawU;

begin
     setcolor(color);
     arc(trunc(x + size / 2), trunc(y + size * YDIVX - size / 2), 180, 360, trunc(size / 2));
     line(x + size, y, x + size, trunc(y + size * YDIVX - size / 2));
     line(x, y, x, trunc(y + size * YDIVX - size / 2));
end; {DrawU}

{*********************** DrawV ***************************}

procedure FontObject.DrawV;

begin
     setcolor(color);
     line(x, y, x + size div 2, trunc(y + size * YDIVX));
     line(x + size, y, x + size div 2, trunc(y + size * YDIVX));
end; {DrawV}

{*************************** DrawW ************************}

procedure FontObject.DrawW;

begin
     setcolor(color);
     line(x, y, x + size div 4, trunc(y + size * YDIVX));
     line(x + size, y, x + 3 * size div 4, trunc(y + size * YDIVX));
     line(x + size div 2, trunc(y + size * YDIVX / 2), x + size div 4, trunc(y + size * YDIVX));
     line(x + size div 2, trunc(y + size * YDIVX / 2), x + 3 * size div 4, trunc(y + size * YDIVX));
end; {DrawW}

{*********************** DrawX **************************}

procedure FontObject.DrawX;

begin
     setcolor(color);
     line(x, y, x + size, trunc(y + size * YDIVX));
     line(x, trunc(y + size * YDIVX), x + size, y);
end; {DrawX}

{*********************** DrawY *************************}

procedure FontObject.DrawY;

begin
     setcolor(color);
     arc(trunc(x + size / 2), trunc(y + size * YDIVX / 2 - size / 2), 180, 0, size div 2);
     line(trunc(x + size / 2), trunc(y + size * YDIVX / 2), x + size div 2, trunc(y + size * YDIVX));
     line(x, y, x, trunc(y + size * YDIVX / 2 - size / 2));
     line(x + size, y, x + size, trunc(y + size * YDIVX / 2 - size / 2));
end; {DrawY}

{************************** DrawZ ****************************}

procedure FontObject.DrawZ;

begin
     setcolor(color);
     line(x, y, x + size, y);
     line(x, trunc(y + size * YDIVX), x + size, trunc(y + size * YDIVX));
     line(x + size, y, x, trunc(y + size * YDIVX));
end; {DrawZ}

{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
            CHARACTER DRAWING ROUTINES STOP HERE
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}

{***************************** Init Graph Letts ***************************}

procedure FontObject.Init;
{init procedure for object}

var ErCd, GrMd, GrDr : integer;   {3 vars used to install BGI drivers}
    par : paratype;               {used to display credits}

    procedure Abort(Msg : string);  {quits}

    begin
         Writeln(Msg, ': ', GraphErrorMsg(GraphResult));
         Halt(1);          {Shows message and quits with a DOS exit code of 1}
    end;

begin
     {Registers all the drivers}
     if RegisterBGIdriver(@CGADriverProc) < 0 then
        Abort('CGA');
     if RegisterBGIdriver(@EGAVGADriverProc) < 0 then
        Abort('EGA/VGA');
     if RegisterBGIdriver(@HercDriverProc) < 0 then
        Abort('Herc');
     if RegisterBGIdriver(@ATTDriverProc) < 0 then
        Abort('AT&T');
     if RegisterBGIdriver(@PC3270DriverProc) < 0 then
        Abort('PC 3270');


     {Registers all the fonts}
     if RegisterBGIfont(@GothicFontProc) < 0 then
        Abort('Gothic');
     if RegisterBGIfont(@SansSerifFontProc) < 0 then
        Abort('SansSerif');
     if RegisterBGIfont(@SmallFontProc) < 0 then
        Abort('Small');
     if RegisterBGIfont(@TriplexFontProc) < 0 then
        Abort('Triplex');

     GrDr := Detect;                  {autodetects the hardware}
     InitGraph(GrDr, GrMd, '');  {activates graphics}
     ErCd := GraphResult;        {finds the error code(s)}
     if ErCd <> GrOK then begin      {if there are any errors...}
        writeln('Graphics error: ', GraphErrorMsg(ErCd));
        writeln('Press ENTER...');
        writeln('Exiting...');    {Outputs message and quits}
        readln;
        halt;
     end; {if}

     YDIVX := YDX;      {sets all private vars. to the values passed}
     tablength := tabl;
     alarmbellfrequency := abf;
     alarmbelltime := abt;

     movetextcursor(2, 2);   {moves text cursor to (2, 2)}
     SetCurrColor(12);       {sets text color to light red}
     SetCurrSize(10);        {sets text size to 10 pixels wide}

end; {Initletters}

{************************ MoveTextCursor *************************}

procedure FontObject.movetextcursor;
{moves text cursor to the values passed}

begin
     textcursorx := x;   {sets private vars. to the passed values}
     textcursory := y;
end;

{************************* GetTextcursorXY *****************************}

procedure FontObject.GetTextCursorXY;
{returns the values of the private vars. of the text position}

begin
     x := textcursorx; {Assigns the vals. of the var. params. to the values}
     y := textcursory; { of the private vars.}
end;

{***************************** Set Current Size *****************************}

procedure FontObject.SetCurrSize;
{sets the value of the private var. size to the value passed}

begin
     CurrentSize := size; {assigns the value of the private var. to the passed value}
end;

{********************** Set Current Color ************************}

procedure FontObject.SetCurrColor;
{sets the value of the private var. color to the value passed}

begin
     CurrentColor := color; {assigns the value of the private var. to the passed value}
end;

{*************************** Get Current Size ***************************}

procedure FontObject.GetCurrSize;
{returns the value of the private var. size}

begin
     size := CurrentSize; {assigns the value of the var. param. to the value of the private var.}
end;

{************************ Get Current Color ****************************}

procedure FontObject.GetCurrColor;
{returns the text color in the var. param}

begin
     color := CurrentColor; {sets the value of the var. param. to the value of the private var.}
end;

{******************************** Destruct *********************************}

procedure FontObject.Destruct;
{cleans up after using the font.  Always call this procedure at the end of the program}

begin
     closegraph;         {exits graphics mode}
     textcolor(15);      {sets normal text color to bright white}
     textbackground(0);  {sets normal text background to black}
end;


{************************** Ps Printf ******************************}

procedure FontObject.psprintf;
{writes a string of characters to the screen.  Pass it a null terminated string}

var ctr, ctr1 : integer;   {loop counters}
    xpos, ypos : integer;  {position counters}
    backslash : boolean;   {backslash boolean flag}
    quote : boolean;       {quote boolean flag}
    size, color : word;    {size & color vars}

begin
     ctr := 1;             {inits counter to 1}
     if ((x >= 0) and (y >= 0)) then begin
        xpos := x;            {if new vals. are passed...}
        ypos := y;
     end {if}
     else begin        {if neg. vals. are passed, then private vals. are used}
         GetTextCursorXY(xpos, ypos);
         GetTextCursorXY(x, y);
     end; {else}
     if (col < 0) then      {if neg. vals. are passed, then private vals. are used}
        GetCurrColor(color)
     else            {if new vals. are passed...}
         color := col;
     if (siz >= 0) then  {if new vals are passed...}
        size := siz
     else              {if neg. vals. are passed, then private vals. are used}
         GetCurrSize(size);
     backslash := false;   {inits. backslash boolean flag to false}
     quote := false;       {inits. quote boolean flag to false}
     while (chrstr[ctr] in [#32..#126]) and (ctr <= ord(chrstr[0])) do begin {Checks if char. is valid}
           case UpCase(chrstr[ctr]) of       {Branches based on val. of char.  Draws the appropriate char.}
                'A' : if backslash then begin
                         sound(ALARMBELLFREQUENCY);
                         delay(ALARMBELLTIME);
                         nosound;
                         xpos := xpos - size - size div 3 - 1;
                         backslash := false;
                      end
                      else
                          DrawA(xpos, ypos, size, color);
                'B' : DrawB(xpos, ypos, size, color);
                'C' : DrawC(xpos, ypos, size, color);
                'D' : DrawD(xpos, ypos, size, color);
                'E' : DrawE(xpos, ypos, size, color);
                'F' : DrawF(xpos, ypos, size, color);
                'G' : DrawG(xpos, ypos, size, color);
                'H' : DrawH(xpos, ypos, size, color);
                'I' : DrawI(xpos, ypos, size, color);
                'J' : DrawJ(xpos, ypos, size, color);
                'K' : DrawK(xpos, ypos, size, color);
                'L' : DrawL(xpos, ypos, size, color);
                'M' : DrawM(xpos, ypos, size, color);
                'N' : if backslash then begin
                         ypos := ypos + trunc(size * YDIVX) + size div 2 + 1;
                         xpos := xpos - size - size div 3 - 1;
                         backslash := false;
                      end
                      else
                          DrawN(xpos, ypos, size, color);
                'O' : DrawO(xpos, ypos, size, color);
                'P' : DrawP(xpos, ypos, size, color);
                'Q' : DrawQ(xpos, ypos, size, color);
                'R' : if backslash then begin
                         ypos := ypos + trunc(size * YDIVX) + size div 2 + 1;
                         xpos := x mod (size + size div 3 + 1) - size - size div 3 - 1;
                         backslash := false;
                      end
                      else
                          DrawR(xpos, ypos, size, color);
                'S' : DrawS(xpos, ypos, size, color);
                'T' : if backslash then begin
                         xpos := xpos + (size + size div 3 + 1) * (TABLENGTH - 1);
                         backslash := false;
                      end
                      else
                          DrawT(xpos, ypos, size, color);
                'U' : DrawU(xpos, ypos, size, color);
                'V' : DrawV(xpos, ypos, size, color);
                'W' : DrawW(xpos, ypos, size, color);
                'X' : DrawX(xpos, ypos, size, color);
                'Y' : DrawY(xpos, ypos, size, color);
                'Z' : DrawZ(xpos, ypos, size, color);
                '0' : Draw0(xpos, ypos, size, color);
                '1' : Draw1(xpos, ypos, size, color);
                '2' : Draw2(xpos, ypos, size, color);
                '3' : Draw3(xpos, ypos, size, color);
                '4' : Draw4(xpos, ypos, size, color);
                '5' : Draw5(xpos, ypos, size, color);
                '6' : Draw6(xpos, ypos, size, color);
                '7' : Draw7(xpos, ypos, size, color);
                '8' : Draw8(xpos, ypos, size, color);
                '9' : Draw9(xpos, ypos, size, color);
                ' ' : DrawSPC(xpos, ypos, size, color);
                '!' : DrawEXC(xpos, ypos, size, color);
                '#' : DrawNUM(xpos, ypos, size, color);
                '?' : DrawQUES(xpos, ypos, size, color);
                '$' : DrawDLR(xpos, ypos, size, color);
                '%' : if backslash then begin
                         DrawPRC(xpos, ypos, size, color);
                         backslash := false;
                      end;
                '(' : DrawOPR(xpos, ypos, size, color);
                ')' : DrawCPR(xpos, ypos, size, color);
                '*' : if backslash then begin
                         DrawTIMES(xpos, ypos, size, color);
                         backslash := false;
                      end
                      else
                          DrawAST(xpos, ypos, size, color);
                '"' : begin
                      if backslash then begin
                         quote := false;
                         xpos := xpos - size - size div 3 - 1;
                         backslash := false;
                      end
                      else
                          case quote of
                           false : begin
                                        DrawODBQT(xpos, ypos, size, color);
                                        quote := true;
                                   end;
                           true  : begin
                                        DrawCDBQT(xpos, ypos, size, color);
                                        quote := false;
                                   end;
                          end;
                      end;
                '''' : DrawCSIQT(xpos, ypos, size, color);
                '`' : DrawOSIQT(xpos, ypos, size, color);
                '+' : DrawPLS(xpos, ypos, size, color);
                ',' : DrawCMA(xpos, ypos, size, color);
                '-' : DrawMNS(xpos, ypos, size, color);
                '.' : DrawPRD(xpos, ypos, size, color);
                '/' : if backslash then begin
                         DrawDIV(xpos, ypos, size, color);
                         backslash := false;
                      end
                      else
                          DrawSLSH(xpos, ypos, size, color);
                ':' : DrawCLN(xpos, ypos, size, color);
                ';' : DrawSMICLN(xpos, ypos, size, color);
                '<' : if backslash then
                         DrawLESOREQ(xpos, ypos, size, color)
                      else
                         DrawLES(xpos, ypos, size, color);
                '>' : if backslash then
                         DrawGREOREQ(xpos, ypos, size, color)
                      else
                          DrawGRE(xpos, ypos, size, color);
                '=' : if backslash then
                         DrawAPPROX(xpos, ypos, size, color)
                      else
                          DrawEQU(xpos, ypos, size, color);
                '~' : if backslash then
                         DrawTIL(xpos, ypos, size, color)
                      else
                          DrawSIM(xpos, ypos, size, color);
                '[' : DrawOBR(xpos, ypos, size, color);
                ']' : DrawCBR(xpos, ypos, size, color);
                '{' : DrawOBRACE(xpos, ypos, size, color);
                '}' : DrawCBRACE(xpos, ypos, size, color);
                '^' : DrawCAR(xpos, ypos, size, color);
                '_' : DrawUND(xpos, ypos, size, color);
                '\' : if backslash then begin
                         DrawBSH(xpos, ypos, size, color);
                         backslash := false;
                      end
                      else begin
                          backslash := true;
                          xpos := xpos - size - size div 3 - 1;
                      end;
           end; {case}
           xpos := xpos + size + size div 3 + 1; {increments x position by appropriate value}
           if xpos > GetMaxX - size then begin  {carriage return if the string exceeds the length of the screen}
              xpos := x mod (size + size div 3 + 1);
              ypos := ypos + trunc(size * YDIVX) + size div 2 + 1;
           end; {if}
           ctr := ctr + 1;  {increments counter by 1}
     end; {while}
     for ctr1 := 1 to (ctr + 1) do  {clears whole string}
         chrstr[ctr1] := #0;

     movetextcursor(xpos, ypos);  {refreshes private vars.}
     SetCurrColor(color);
     SetCurrSize(size);

end;

{***************** Flash Paragraph *********************}

procedure FontObject.psflashpara;
{Outputs a paragraph in 'Mission Impossible' font}

var xpos, ypos : integer;   {position counters}
    ctr1, ctr2, ctr3 : integer;   {loop counters}

begin
     xpos := x;   {sets position counters to the values passed}
     ypos := y;
     for ctr1 := 1 to rows do begin  {Draws the specified num. of lines}
         for ctr2 := 1 to cols do begin {Draws an entire line}
             psprintf(xpos, ypos, siz, col+8, 1, para[ctr1][ctr2]+#0); {draws the char}
             delay(delaytime);           {pauses for the specified amount of time}
             if ctr2 > tail_length then
                psprintf(xpos - xin * tail_length, ypos, siz, col, 1, para[ctr1][ctr2-tail_length]+#0); {draws the char right
                                                                                                        before the former char}
             xpos := xpos + xin {Increments the x position by the specified amount}
         end; {for}
         xpos := xpos - xin * (tail_length);
         for ctr3 := (ctr2 - tail_length + 1) to cols do begin
             psprintf(xpos, ypos, siz, col, 1, para[ctr1][ctr3]+#0); {draws the last
                                                                      characters again}
             xpos := xpos + xin;
             delay(delaytime)
         end;
         xpos := x;  {carriage return to original column}
         ypos := ypos + yin  {y position is incremented by specified amount}
     end;  {for}
end;


{************************* Ps Scanf *************************}

function FontObject.psscanf;
{reads in values}

var int : integer;
    stringvar : string;
    realnum : real;
    longnum : longint;
    integerreturn : integer absolute ptr;  {Open parameters need absolute}
    stringreturn : string absolute ptr;    { vars.}
    realreturn : real absolute ptr;
    longreturn : longint absolute ptr;

{====================== Ps Scan Int ================================}

function psscanint(var integertpnum : integer) : byte;
{reads in an integer}

var number : integer;       {total}
    digit : char;           {current digit entered}
    plusorminus : shortint; {flag}

begin
     psscanint := 0;    {assigns 'no error' to the return value}
     digit := ReadKey;  {assigns digit to the key just read}
     number := 0;       {sets the total to 0}
     plusorminus := 1;  {sets flag to positive}
     case digit of      {checks if digit is a sign}
          #45 : begin                     {'-' sign...}
                plusorminus := -1;        {sets flag to negative}
                psprintf(-1, -1, -1, -1, 1, digit);  {writes char. on screen}
                digit := ReadKey;          {reads in a new value for digit}
                end;
          #43 : begin                  {'+' sign...}
                plusorminus := 1;        {sets flag to positive}
                psprintf(-1, -1, -1, -1, 1, digit); {echoes char. on screen}
                digit := ReadKey;             {reads in a new value for digit}
                end;
     end; {case}
     while (digit in [#48..#57]) do begin  {repeats until the digit is non-numeric}
           if ((number > 3276) or (number < -3276)) then begin {if the total overflows}
              integertpnum := 0;       {sets return val. to 0}
              psscanint := 1;        {returns error code to calling module}
              exit;                {exits out of procedure}
           end  {if}
           else begin
                number := number * 10 + plusorminus * (ord(digit) - ord('0'));  {adds the digit to the total}
                psprintf(-1, -1, -1, -1, 1, digit);   {echoes char. on screen}
                digit := ReadKey;    {reads in new value for digit}
           end  {else}
     end; {while}

     integertpnum := plusorminus * number; {returns the number to the calling module}

end;

{======================== Ps Scan Long ===================================}

function psscanlong(var longtpnum : longint) : byte;
{reads in a long integer}

var number : longint;       {total}
    digit : char;           {current digit}
    plusorminus : shortint; {sign flag}

begin
     digit := ReadKey;      {reads in a char. and puts it in the curr. digit}
     number := 0;           {inits. total to 0}
     plusorminus := 1;      {sets sign flag to positive}
     case digit of          {checks if digit is a sign}
          #45 : begin       {if it is '-'...}
                plusorminus := -1;        {sets sign flag to negative}
                psprintf(-1, -1, -1, -1, 1, digit); {echoes char. on screen}
                digit := ReadKey;         {reads in new val. for curr. char.}
                end;
          #43 : begin         {if it is '+'...}
                plusorminus := 1;     {sets sign flag to positive}
                psprintf(-1, -1, -1, -1, 1, digit); {echoes char. on screen}
                digit := ReadKey;      {reads in new val. for curr. char.}
                end;
          else ;      {otherwise, do nothing}
     end; {case}
     while (digit in [#48..#57]) do begin {repeat while char. is a digit}
           if ((number > 214748364) or (number < -214748364)) then begin
              longtpnum := 0; {returns a 0}                    {overflow}
              psscanlong := 1; {sets error code to error}
              exit;     {returns to calling module}
           end  {if}
           else begin
                number := number * 10 + ord(digit) - ord('0'); {adds char. to total}
                psprintf(-1, -1, -1, -1, 1, digit); {echoes char. on screen}
                digit := ReadKey;  {reads in new val. for curr. char.}
           end;  {else}
     end; {while}

     psscanlong := 0;   {sets error code to none}
     longtpnum := plusorminus * number;  {returns the total to calling module}

end;

{============================== Ps Scan Str ================================}

function psscanstr(var strtpnum : string) : byte;
{inputs a string}

var stringtoret : string; {string total}
    len : word;           {length counter}
    letter : char;        {current letter}

begin
     stringtoret[0] := #0;   {inits. total to 0}
     letter := ReadKey;      {reads in char. for curr. letter}
     len := 0;               {inits. length counter to 0}
     while letter in [#32..#126] do begin  {repeats until char. isn't valid}
           if (len < 255) then begin  {no overflow...}
              if (letter = '\') then  {if the char. is a '\'}
                 psprintf(-1, -1, -1, -1, 1, '\\');  {echoes '\' on screen}
              psprintf(-1, -1, -1, -1, 1, letter);   {echoes it on screen}
              len := len + 1;         {increments length counter by one}
              stringtoret[len] := Upcase(letter); {adds char. to total}
              letter := ReadKey;
           end  {if}
           else begin  {overflow!}
               psscanstr := 1;  {sets error code to error}
               psprintf(-1, -1, -1, -1, 1, 'OVERFLOW ERROR!'); {outputs 'overflow error!'}
               exit; {returns to calling module}
           end; {else}
     end; {while}
     psscanstr := 0;      {sets error code to no error}
     stringtoret[0] := chr(len);  {sets index 0 to string len.}
     stringtoret[len + 1] := #0;  {terminates with a null zero}
     strtpnum := stringtoret;     {returns the inputted string}

end;

{======================== Ps Scan Real ==============================}

function psscanreal(var realtpnum : real) : byte;
{reads in a real number}

var realtoreturn : real;     {total}
    stringreal : string;     {real in string form}
    errcd : integer;         {error flag}

begin
     realtpnum := 0;       {sets total to 0}
     psscanstr(stringreal);  {reads in the real as a string}
     Val(stringreal, realtoreturn, errcd);{converts the string back to a real}
     if (errcd <> 0) then    {if there is an error...}
        psscanreal := errcd      {sets error code to error}
     else                {No error!}
         psscanreal := 0;   {sets error code to no error}
     realtpnum := realtoreturn;  {returns the real number total}
end;

{++++++++++++++++++++++++++++ Ps Scanf Main ++++++++++++++++++++++++++++++++}

begin
     psscanf := 0;   {sets error code to no error}
     case scantype[2] of   {determines which data type will be inputted}
          'l' : begin
                if (psscanlong(longnum) = 1) then {if there was an error...}
                   psscanf := 1     {sets error code to error}
                else    {No error!}
                    longreturn := longnum; {returns the value}
                end;  {if-else}
          'd' : begin
                if (psscanint(int) = 1) then {if there was an error...}
                   psscanf := 1 {sets error code to error}
                else   {No error!}
                    integerreturn := int;   {returns the value}
                end;  {if-else}
          'r' : begin
                if (psscanreal(realnum) = 1) then  {if there was an error...}
                   psscanf := 1  {sets error code to error}
                else  {No error!}
                    realreturn := realnum;   {returns the value}
                end; {if-else}
          's' : begin
                if (psscanstr(stringvar) = 1) then {if there was an error...}
                   psscanf := 1   {sets error code to error}
                else   {No error!}
                    stringreturn := stringvar;  {returns the value}
                end; {if-else}
     end; {case}
end;

{*********************** Flash Scan Function ************************}

function FontObject.psFlashScanf;

{Inputs a string, less than 255 chars. long}

var xpos, ypos : integer; {Pixel position vars.}
    lasttime : record {Records the last time the cursor flashed}
               hr, mn, sc, sc100 : longint {time vars.}
               end; {record}
    timeelapsed : longint; {Records the number of 1/100 seconds elapsed since
                            last flash}
    on, switch : boolean; {Flags which control the flashing of the cursor}
    time : record {Records the time at the moment}
           hr, mn, sc, sc100 : longint {time vars.}
           end; {record}
    key : char; {The key which the user pressed}
    tmpstr : string; {Temporary string var.  Used to store the input string
                      and is passed back to the calling module at the end}
    length : byte; {Contains the length of the temp. string}
    pos : byte; {Contains the position of the cursor in the temp. string}
    InsFlag : boolean; {Flag which records whether or not the reading function
                        is in insert mode or overwrite mode.}
    errorcode : byte; {Contains the error code to be passed back to the
                       calling module}

    procedure InsChar(var tmpstr : string; pos : byte; ch : char);

    {inserts a character into the passed string at the passed position}

    var
       ctr : integer; {loop counter}
       tmpchar1, tmpchar2 : char; {Two temp. chars. used to copy the contents
                                   of the passed string forward one position}

    begin {procedure}
         inc(tmpstr[0]); {Increments the length byte of the passed string by
                          1 unit}
         tmpchar1 := tmpstr[pos]; {Sets one temp. char. to the char. at the
                                   passed position}
         tmpstr[pos] := ch; {Sets the char. at the passed position to the
                             passed char.}
         for ctr := pos + 1 to ord(tmpstr[0]) do begin
             tmpchar2 := tmpstr[ctr]; {Sets another temp. char. to the char.
                                       at the current position.}
             tmpstr[ctr] := tmpchar1; {Sets the char. at the current position
                                       to the char. at the prev. position}
             tmpchar1 := tmpchar2 {Sets a temp. char. to the char. at the
                                   current position}
         end {for}
    end; {procedure}

    procedure DelChar(var tmpstr : string; pos : byte);

    {deletes the char. in the passed string at the passed position}

    var
       ctr : integer; {loop counter}

    begin {procedure}
         for ctr := pos to ord(tmpstr[0]) - 1 do
             tmpstr[ctr] := tmpstr[ctr + 1]; {Copies the char. at the next
                                              position to the current pos.}
         dec(tmpstr[0]) {Decrements the length byte of the passed string by 1}
    end; {procedure}

begin {function}
     xpos := x; {inits the x pos. ctr. to the passed start x value}
     ypos := y; {inits the y pos. ctr. to the passed start y value}
     timeelapsed := 1000; {Sets the time elapsed ctr. to 1000}
     on := true; {Turns the cursor on.}
     switch := true; {Sets the switch flag to false}
     length := 1; {Inits the length ctr. to 1}
     tmpstr[0] := #1; {Sets the temp. string to a null char.}
     tmpstr[1] := #0;
     pos := 1; {Sets the position ctr to 1}
     InsFlag := true; {turns on insert mode}
     key := #1;       {Sets the key pressed to char. 1}
     errorcode := 0; {Sets error code to none}
     repeat  {Inputs the string}
           if switch then begin   {Controls the flashing of the cursor}
              if on then begin  {If the cursor is to be turned off...}
                 setcolor(0);
                 setfillstyle(solidfill, 0)
              end
              else begin
                   setfillstyle(solidfill, col);
                   setcolor(col)
              end;
              if InsFlag then begin
                 line(xpos, ypos + trunc(YDIVX * siz), xpos + siz, ypos + trunc(YDIVX * siz));
                 line(xpos, ypos + trunc(YDIVX * siz) - 1, xpos + siz, ypos + trunc(YDIVX * siz) - 1)
              end
              else begin
                   bar(xpos, ypos, xpos + siz, ypos + trunc(YDIVX * siz))
              end;
              if on then
                 psprintf(xpos, ypos, siz, col, 1, tmpstr[pos]);
              on := not(on);
              switch := false;
              with lasttime do
                   pgettime(hr, mn, sc, sc100)
           end;
           with time do
                pgettime(hr, mn, sc, sc100);
           with time do
                timeelapsed := 360000*hr + 6000*mn + 100*sc + sc100;
           with lasttime do
                timeelapsed := timeelapsed - (360000*hr + 6000*mn + 100*sc + sc100);
           if timeelapsed >= timelapse then
              switch := true;
           if keypressed then
              key := readkey;
           if key = #0 then begin
              key := readkey;
              if key = #75 then begin
                 if pos > 1 then begin
                    drawSPC(xpos, ypos, siz, col);
                    psprintf(xpos, ypos, siz, col, 1, tmpstr[pos]+#0);
                    xpos := xpos - siz - siz div 3 - 1;
                    if xpos < 0 then begin
                       xpos := (GetMaxX - x) div (siz + siz div 3 + 1) * (siz + siz div 3 + 1) + x;
                       ypos := ypos - trunc(YDIVX * siz) - siz div 2 - 1
                    end;
                    pos := pos - 1
                 end;
                 key := #1
              end
              else if key = #77 then begin
                   if pos < length then begin
                      drawSPC(xpos, ypos, siz, col);
                      psprintf(xpos, ypos, siz, col, 1, tmpstr[pos]+#0);
                      xpos := xpos + siz + siz div 3 + 1;
                      if xpos > GetMaxX - siz - 1 then begin
                         xpos := x mod (siz + siz div 3 + 1);
                         ypos := ypos + trunc(YDIVX * siz) + siz div 2 + 1
                      end;
                      pos := pos + 1
                   end;
                   key := #1
              end                      { NOT IMPLEMENTED YET
              else if key = #71 then begin
                   drawSPC(xpos, ypos, siz, col);
                   psprintf(xpos, ypos, siz, col, 1, tmpstr[pos]+#0);
                   xpos := x;
                   ypos := y;
                   pos := 1;
                   key := #1
              end
              else if key = #79 then begin
                   drawSPC(xpos, ypos, siz, col);
                   psprintf(xpos, ypos, siz, col, 1, tmpstr[pos]+#0);
                   length := length - 1;
                   if length > (GetMaxX - x) div (siz + siz div 3 + 1) then
                      xpos := ((length - 1) mod (trunc((GetMaxX - x) / (siz + siz div 3 + 1)))) * (siz + siz div 3 + 1) + 2
                   else
                       xpos := ((length - 1) mod (trunc((GetMaxX - x) / (siz + siz div 3 + 1)))) * (siz + siz div 3 + 1) + x;
                   if length > (GetMaxX - x) div (siz + siz div 3 + 1) then
                      ypos := y + ((length - (GetMaxX - x) div (siz + siz div 3 + 1)) div (x div (siz + siz div 3 + 1) +
                              (GetMaxX - x) div (siz + siz div 3 + 1)) + 1) * (trunc(YDIVX * siz) + siz div 2 + 1)
                   else
                       ypos := y;
                   length := length + 1;
                   pos := length;
                   key := #1
              end             }      {NOT IMPLEMENTED YET}
              else if key = #82 then begin
                   if on then begin
                      drawSPC(xpos, ypos, siz, 0);
                      psprintf(xpos, ypos, siz, col, 1, tmpstr[pos])
                   end;
                   InsFlag := not(InsFlag);
                   key := #1
              end
              else if key = #83 then begin
                   if ord(tmpstr[0]) - pos > 0 then begin
                      psprintf(x, y, siz, 0, 1, tmpstr+#0);
                      Delchar(tmpstr, pos);
                      length := length - 1;
                      tmpstr[0] := chr(length);
                      psprintf(x, y, siz, col, 1, tmpstr+#0)
                   end;
                   key := #1
              end
              else
                  key := #1
           end
           else if key = #8 then begin
                if pos > 1 then begin
                   drawSPC(xpos, ypos, siz, col);
                   xpos := xpos - siz - siz div 3 - 1;
                   if xpos < 0 then begin
                      xpos := (GetMaxX - x) div (siz + siz div 3 + 1) * (siz + siz div 3 + 1) + x;
                      ypos := ypos - trunc(YDIVX * siz) - siz div 2 - 1
                   end;
                   pos := pos - 1;
                   length := length - 1;
                   psprintf(x, y, siz, 0, 1, tmpstr+#0);
                   Delchar(tmpstr, pos);
                   tmpstr[0] := chr(length);
                   psprintf(x, y, siz, col, 1, tmpstr+#0)
                end;
                key := #1
           end
           else if key = #13 then begin
           end
           else if key in [#32..#126] then begin
                if pos = length then begin
                   InsChar(tmpstr, pos, key);
                   pos := pos + 1;
                   length := length + 1;
                   psprintf(xpos, ypos, siz, col, 1, ' ');
                   psprintf(xpos, ypos, siz, col, 1, key+#0)
                end
                else begin
                     if not(InsFlag) then begin
                        drawSPC(xpos, ypos, siz, 0);
                        tmpstr[pos] := key;
                        psprintf(xpos, ypos, siz, col, 1, tmpstr[pos]);
                        pos := pos + 1
                     end
                     else begin
                          drawSPC(xpos, ypos, siz, 0);
                          psprintf(x, y, siz, 0, 1, tmpstr);
                          InsChar(tmpstr, pos, key);
                          pos := pos + 1;
                          length := length + 1;
                          psprintf(x, y, siz, col, 1, tmpstr)
                     end
                end;
                xpos := xpos + siz + siz div 3 + 1;
                if xpos > GetMaxX - siz - 1 then begin
                   xpos := x mod (siz + siz div 3 + 1);
                   ypos := ypos + trunc(YDIVX * siz) + siz div 2 + 1
                end;
                key := #1
           end;
           if tmpstr[0] = #255 then begin
              errorcode := 255;
              break
           end
     until key = #13;
     drawSPC(xpos, ypos, siz, 0);
     psprintf(xpos, ypos, siz, col, 1, tmpstr[pos]);
     tmpstr[0] := chr(length);
     returnstr := tmpstr;
     psflashscanf := errorcode;
end;                                               {

<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                         END CONSTRUCTION
<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}


{~~~~~~~~~~~~~~~~~~~~~~~~ End of FontObject ~~~~~~~~~~~~~~~~~~~~~~~}

{~~~~~~~~~~~~~~~~~~~~~~~ Conversion Routines ~~~~~~~~~~~~~~~~~~~~~}

{********************** Converts Longs To Strings ************************}

function LongToString(longtostrnum : longint) : string;

var ctr, len : byte;   {position counter and length counter}
    stringtoreturn : string; {return string}
    longtmp : longint; {temp. long}
    plus : boolean;    {sign flag}

begin
     if (longtostrnum >= 0) then begin {if it is positive...}
        plus := true;  {sets sign flag to positive}
        ctr := 0       {sets position counter to 0}
     end  {if}
     else begin
         plus := false; {sets sign flag to negative}
         ctr := 1;      {sets position counter to 1}
     end;  {else}
     longtmp := longtostrnum; {sets temp. long to the passed value}
     while (longtmp <> 0) do begin  {while the temp. long is not 0}
           ctr := ctr + 1; {increments counter}
           longtmp := (longtmp - longtmp mod 10) div 10; {adds current digit to return string}
     end; {while}
     len := ctr;   {sets length counter to the position counter}
     longtmp := longtostrnum; {sets temp. long to the passed value}
     if plus then  {if it is positive...}
        for ctr := len downto 1 do begin   {places the digits in the string}
            stringtoreturn[ctr] := chr(longtmp mod 10 + 48);
            longtmp := (longtmp - longtmp mod 10) div 10;
        end {for}
     else begin
         stringtoreturn[1] := '-';   {puts a '-' in the front of the string}
         for ctr := len downto 2 do begin    {places the digits in the string}
            stringtoreturn[ctr] := chr(48 - (longtmp mod 10));
            longtmp := (longtmp - longtmp mod 10) div 10;
         end; {for}
     end;  {else}
     stringtoreturn[0] := chr(len);  {places the length in the first location}
     stringtoreturn[len + 1] := #0;  {terminates with a null character}
     if stringtoreturn[0] = #0 then
        begin
             Inc(stringtoreturn[0]);
             stringtoreturn[1] := '0'
        end;
     LongToString := stringtoreturn; {places the string in the return value}
end;

{***************** Converts Integers to Strings ****************}

function IntToString(inttoconvert : integer) : string;

var strtoreturn : string;    {return string total}

begin
     str(inttoconvert, strtoreturn);  {converts integer to string}
     IntToString := strtoreturn;      {returns the string}
end;

end.  {unit}